/*
    Copyright (C) 2013 Daniel Vrátil <dvratil@redhat.com>

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
    License as published by the Free Software Foundation; either
    version 2.1 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public
    License along with this library; if not, write to the Free Software
    Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
*/

#include "message.h"

#include <QtCore/QSharedData>

namespace KTp
{

class Message::Private : public QSharedData {

  public:
    Private() :
        QSharedData(),
        isHistory(false)
    {}

    QDateTime   sentTime;
    QString     token;
    Tp::ChannelTextMessageType messageType;
    QVariantMap properties;
    QString     mainPart;
    QStringList parts;
    QStringList scripts;
    bool isHistory;
    MessageDirection direction;

    //if we have a valid sender pointer store that, otherwise store alias
    KTp::ContactPtr sender;
    QString senderAlias;
    QString senderId;
};

}
