/*
 * Copyright (C) 2012 Dan Vrátil <dvratil@redhat.com>
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 */

#include "logs-importer.h"
#include "logs-importer-private.h"

#include <TelepathyQt/Account>

using namespace KTp;

LogsImporter::LogsImporter(QObject *parent)
  : QObject(parent)
  , d(new LogsImporter::Private(this))
{
    connect(d, SIGNAL(finished()), SIGNAL(logsImported()));
    connect(d, SIGNAL(error(QString)), SIGNAL(error(QString)));
}

LogsImporter::~LogsImporter()
{
    delete d;
}

bool LogsImporter::hasKopeteLogs(const Tp::AccountPtr& account)
{
    return !d->findKopeteLogs(account->uniqueIdentifier()).isEmpty();
}

void LogsImporter::startLogImport(const Tp::AccountPtr& account)
{
    d->setAccountId(account->uniqueIdentifier());
    d->start();
}
