/*
 * Copyright (C) 2014 Amandeep Singh <aman.dedman@gmail.com>
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 */


#ifndef CONTACT_GRID_WIDGET_H
#define CONTACT_GRID_WIDGET_H

#include <KTp/Widgets/contact-view-widget.h>

namespace KTp
{

class KTPCOMMONINTERNALS_EXPORT ContactGridWidget : public ContactViewWidget
{
    Q_OBJECT
    Q_DISABLE_COPY(ContactGridWidget)

public:
    explicit ContactGridWidget(ContactsListModel *model, QWidget *parent = nullptr);

}; // class ContactGridWidget

} // namespace KTp

#endif // CONTACT_GRID_WIDGET_H
