/*
 * Settings KCM Dialog
 *
 * Copyright (C) 2011 Martin Klapetek <martin.klapetek@gmail.com>
 * Copyright (C) 2014 Siddhartha Sahu <sh.siddhartha@gmail.com>
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 */

#ifndef SETTINGS_KCM_DIALOG_H
#define SETTINGS_KCM_DIALOG_H

#include <ksettings/Dialog>
#include <KTp/ktpcommoninternals_export.h>

namespace KTp
{

class KTPCOMMONINTERNALS_EXPORT SettingsKcmDialog : public KSettings::Dialog
{
    Q_OBJECT
public:
    SettingsKcmDialog(QWidget *parent = nullptr);

    // Add the General Settings tab
    void addGeneralSettingsModule();
    // Add the Notifications tab
    void addNotificationsModule();
};

} // namespace KTp

#endif // SETTINGS_KCM_DIALOG_H
